##################################################
# Dr. Javier DE LAS RIVAS and Sara AIBAR
# Cancer Research Center (CIC-IBMCC, CSIC/USAL)
# Salamanca - SPAIN
# http://ubioinfo.cicancer.org/en/course10.html
# http://bioinfow.dep.usal.es
##################################################
##################################################
# R PROTOCOL
# Enrichment statistical tests
# HYPERGEOMETRIC test and FISHER EXACT test
##################################################


###################################
###     WORKING DIRECTORY       ###
###################################

# set working directory #
# Windows:
setwd("C:\\...\\Desktop\\...\\Lab") 
# MacOSX:
setwd("/Users/.../Desktop/.../Lab")
# Linux:
setwd("/home/.../Desktop/.../Lab")

# Working directory
setwd("/Volumes/DATOS/xxxCARPETAxxx/")
dir()



###################################
###  HYPERGEOMETRIC test in R   ###
###################################

?phyper
phyper(q, m, n, k, lower.tail=TRUE, log.p=FALSE)
x, 
q vector of quantiles representing the number of WHITE balls drawn without replacement from an urn which contains both BLACK and WHITE balls.
m the number of WHITE balls in the urn.
n the number of BLACK balls in the urn.
k the total number of balls drawn from the urn.
lower.tail  logical; if TRUE (default), probabilities are P[X ≤ x], otherwise, P[X > x]
The SET or QUERY LIST analysed is k
The SUBSET selected is q
The UNIVERSE is m+n

# EXAMPLE (62 g WHITE of 131 g in the sample list ... being 1998 g WHITE in a 5260 g total population) 
Population size : 5260 === ALL
Sample size : 131 === k
Number of items in the population (i.e. the universe) that are classified as successes : 1998 === m
Number of items in the sample (i.e. the query list) that are classified as successes : 62 === x

x=62
k=131
m=1998
n=5260-1998

phyper(x, m, n, k)
[1] 0.9892471

# P(Observed 62 or more) = 1-P(Observed less than 62)

1-phyper(62,1998,(5260-1998),131)
[1] 0.01075292
phyper(62,1998,(5260-1998),131, lower.tail=FALSE)
[1] 0.01075292 **

x <- 62         # red marbles selected
k <- 131        # TOTAL marbles selected
m <- 1998       # red marbles in the urn
n <- 5260-1998  # TOTAL black in the urn
phyper(x, m, n, k, lower.tail=FALSE)
[1] 0.01075292     # It is the same as the FISHER EXACT calculated below


# RUN THIS EXAMPLE (15 g WHITE of 200 g in the sample list ... being 120 g WHITE in a 4000 g total population) 
Population size : 4000 === ALL
Sample size : 200 === k
Number of items in the population (i.e. the universe) that are classified as successes : 120 === m
Number of items in the sample (i.e. the query list) that are classified as successes : 15 === x

phyper(x, m, n, k, lower.tail=FALSE)
phyper(15, 120, (4000-120), 200, lower.tail=FALSE)
[1] 0.0002564379 ***



###################################
###  The SIZE of the population ###
###################################

# The SIZE of the POPULATION matters !!! (with the same EXAMPLE)
P = function(size) 
{
  nFound <- 62          						# x=nFound
  nDrawn <- 131                                 # k=nDrawn
  nTotAtCategory <- 1998                        # m=nTotAtCategory
  nTotNotAtCategory <- (size - nTotAtCategory)  # n=nTotNotAtCategory
  # phyper(x, m, n, k, lower.tail=FALSE)
  phyper(nFound, nTotAtCategory, nTotNotAtCategory, nDrawn, lower.tail=FALSE)
}
P(5260)
[1] 0.01075292 **
P(3260)
[1] 0.9993491


# The SIZE of the POPULATION matters !!! (with a simple EXAMPLE)
P = function(size) 
{
  nFound <- 15          						# x=nFound
  nDrawn <- 200                                 # k=nDrawn
  nTotAtCategory <- 120                         # m=nTotAtCategory
  nTotNotAtCategory <- (size - nTotAtCategory)  # n=nTotNotAtCategory
  phyper(nFound, nTotAtCategory, nTotNotAtCategory, nDrawn, lower.tail=FALSE) 
}
P(4000)
[1] 0.0002564379 ***
P(1000)
[1] 0.9839485



###################################
###    FISHER EXACT test in R   ###
###################################

The "Hypergeometric" test is equivalent to a "one-tailed Fisher Exact" test

The Fixher Test takes always a 2 x 2 contingency table:
a b 
c d 
It interprets the situation under the null hypothesis as taking a 
sample of size a+c from an urn with a+b white balls and c+d black balls. 
a=1
b=2
c=3
d=4
matrix(c(1,3,2,4),2,2)
     [,1] [,2]
[1,]    1    2   # a b
[2,]    3    4   # c d

# EXAMPLE (62 g WHITE of 131 g in the sample list ... being 1998 g WHITE in a 5260 g total population)
Population size : 5260 === ALL
Sample size : 131
Number of items in the population (i.e. the universe) that are classified as successes : 1998 === m
Number of items in the sample (i.e. the query list) that are classified as successes : 62 === x

matrix( c(62,131-62,1998-62,(5260-1998-c(131-62)) ),2,2)
     [,1] [,2]  
[1,]   62 1936   # White.selected  v  White.not_selected
[2,]   69 3193   # Black.selected  v  Black.not_selected
62+1936
[1] 1998  # m the number of white balls in the urn
62+1936+69+3193
[1] 5260  # the total number of balls in the urn


matrix(c(1, 2, 3, 4), 
           nrow = 2,
           dimnames = list(COLOR = c("white", "black"),
                           SELECTION = c("selected", "not_selected")))
       SELECTION
COLOR   selected not_selected
  white        1            3
  black        2            4

matrix( c(62,131-62,1998-62,(5260-1998-c(131-62)) ),
           nrow = 2,
           dimnames = list(COLOR = c("white", "black"),
                           SELECTION = c("selected", "not_selected")))
       SELECTION
COLOR   selected not_selected
  white       62         1936
  black       69         3193

mat <- matrix( c(62,131-62,1998-62,(5260-1998-c(131-62)) ),
           nrow = 2,
           dimnames = list(COLOR = c("white", "black"),
                           SELECTION = c("selected", "not_selected")))

fisher.test(mat, alternative='less')$p.value
[1] 0.9892471                          
# alternative === indicates the alternative hypothesis and must be one of "two.sided", "greater" or "less". You can specify just the initial letter. Only used in the 2 by 2 case.

# P(Observed 62 or more) = 1-P(Observed less than 62)

1 - fisher.test(mat, alternative='less')$p.value
[1] 0.01075292    # It is the same as the HYPERGEOMETRIC calculated above                   
             
                           

##################################################
# END
##################################################
