/*
 * Created on Nov 24, 2003
 */
package org.gk.database;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.gk.model.GKInstance;
import org.gk.persistence.MySQLAdaptor;
import org.gk.persistence.PersistenceManager;
import org.gk.persistence.XMLFileAdaptor;
import org.gk.util.GKApplicationUtilities;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * This customized JDialog is used to create a new _Deleted object.
 * @author croft
 */
public class DeletedInstanceDialog extends JDialog {
    private static boolean isDownloadDefaultParams = false;
    private AttributePane attributePane;
    private boolean isOKClicked;
    private boolean isCancelClicked;
    
    public DeletedInstanceDialog(JFrame parentFrame, GKInstance instance) {
        super(parentFrame, "Deletion information");
        attributePane = new AttributePane(instance);
        isOKClicked = false;
        isCancelClicked = true;
        downloadDefaultParams();
        init();
    }
    
    private void init() {
        // Add instance pane
        attributePane.refresh();
        attributePane.setEditable(true);
        attributePane.setBorder(BorderFactory.createRaisedBevelBorder());
        attributePane.setTitle("Enter information pertaining to deleted instance:");
        getContentPane().add(attributePane, BorderLayout.CENTER);
        // Add buttons
        JPanel buttonPane = new JPanel();
        buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.Y_AXIS));
        JButton okBtn = new JButton("Delete with _Deleted Instance");
        okBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                isOKClicked = true;
                isCancelClicked = false;
                attributePane.stopEditing();
                dispose();
            }
        });
        okBtn.setMnemonic('O');
        okBtn.setAlignmentX(Component.CENTER_ALIGNMENT);
        JButton skipBtn = new JButton("Delete without _Deleted Instance");
        skipBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                isCancelClicked = false;
                attributePane.stopEditing();
                dispose();
            }
        });
        skipBtn.setMnemonic('S');
        skipBtn.setAlignmentX(Component.CENTER_ALIGNMENT);
        JButton cancelBtn = new JButton("Cancel");
        cancelBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
        cancelBtn.setMnemonic('C');
        cancelBtn.setAlignmentX(Component.CENTER_ALIGNMENT);
        okBtn.setDefaultCapable(true);
        okBtn.setMinimumSize(skipBtn.getPreferredSize());
        cancelBtn.setMinimumSize(skipBtn.getPreferredSize());
        buttonPane.add(Box.createRigidArea(new Dimension(5, 5)));
        buttonPane.add(okBtn);
        buttonPane.add(Box.createRigidArea(new Dimension(5, 5)));
        buttonPane.add(skipBtn);
        buttonPane.add(Box.createRigidArea(new Dimension(5, 5)));
        buttonPane.add(cancelBtn);
        buttonPane.add(Box.createRigidArea(new Dimension(5, 5)));
        getContentPane().add(buttonPane, BorderLayout.NORTH);
    }
    
    public boolean isOKClicked() {
        return this.isOKClicked;
    }
    
    public boolean isCancelClicked() {
        return this.isCancelClicked;
    }
    
    private void downloadDefaultParams() {
        // Already done?  Then return.
        if (isDownloadDefaultParams)
            return;
        
        try {
            InputStream metaConfig = GKApplicationUtilities.getConfig("curator.xml");
            if (metaConfig == null)
                return;
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = dbf.newDocumentBuilder();
            Document document = builder.parse(metaConfig);
            Element root = document.getDocumentElement();
            NodeList childNodes = root.getChildNodes();
            int size = childNodes.getLength();
            for (int i = 0; i < size; i++) {
                org.w3c.dom.Node node = childNodes.item(i);
                if (node.getNodeName().equals("deletedInstances")) {
                    NodeList list = node.getChildNodes();
                    for (int j = 0; j < list.getLength(); j++) {
                        org.w3c.dom.Node node1 = list.item(j);
                        if (node1.getNodeName().equals("controlledVocabulary")) {
                            NamedNodeMap attributes = node1.getAttributes();
                            for (int k=0; k<attributes.getLength(); k++) {
                                Node attribute = attributes.item(k);
                                String name = attribute.getNodeName();
                                if (name.equals("instanceClass")) {
                                    String instanceClass = attribute.getNodeValue();
                                    downloadInstances(instanceClass);
                                    break;
                                }
                            }
                        }
                    }
                    isDownloadDefaultParams = true;
                    return;
                }
            }
        }
        catch(Exception e) {
            System.err.println("DeletedInstanceDialog.downloadDefaultParams(): " + e);
            e.printStackTrace();
        }
    }

    private void downloadInstances(String instanceClass) {
        if (instanceClass==null || instanceClass.equals(""))
            return;
        
        XMLFileAdaptor fileAdaptor = PersistenceManager.getManager().getActiveFileAdaptor();
        Collection instances;

        // First check to see if any instances of this
        // class are already present - if so, assume
        // that we don't need to check out anything.
        try {
            instances = fileAdaptor.fetchInstancesByClass(instanceClass);
            if (instances!=null && instances.size()>0) {
                // Some instances already present locally, so don't
                // try to check any out from database.
                return;
            }
        } catch (Exception e) {
            System.err.println("DeletedInstanceDialog.downloadInstances: problem fetching existing " + instanceClass + " instances from local file");
            e.printStackTrace();
        }
        
        MySQLAdaptor dbAdaptor = PersistenceManager.getManager().getActiveMySQLAdaptor(this);

        try {
            instances = dbAdaptor.fetchInstancesByClass(instanceClass);
        } catch (Exception e) {
            System.err.println("DeletedInstanceDialog.downloadInstances: problem fetching existing " + instanceClass + " instances from database");
            e.printStackTrace();
            return;
        }
        if (instances==null)
            return;
        List instancesList = new ArrayList(instances);
        try {
            SynchronizationManager.getManager().checkOut(instancesList, this);
        } catch (Exception e) {
            System.err.println("DeletedInstanceDialog.downloadInstances: problem checking out instances of class " + instanceClass + " from database");
            e.printStackTrace();
        }
    }
}
