/*
 * Created on May 24, 2010
 *
 */
package org.gk.database;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.gk.model.GKInstance;
import org.gk.persistence.PersistenceManager;
import org.gk.persistence.XMLFileAdaptor;
import org.gk.schema.InvalidAttributeException;
import org.gk.schema.InvalidAttributeValueException;
import org.gk.schema.SchemaClass;

/**
 * This class is used to do instance deletion.
 * @author wgm
 *
 */
public class InstanceDeletion {
    
    public InstanceDeletion() {
    }
    
    /**
     * Delete a list of GKInstance.
     * @param list
     * @param parentFrame
     */
    public void delete(List<GKInstance> list,
                       JFrame parentFrame) {
        if (list == null || list.size() == 0)
            return;
        int reply = JOptionPane.showConfirmDialog(parentFrame,
                                                  "Are you sure you want to delete the selected instance" + (list.size() == 1 ? "" : "s") + "? " +
                                                  "The slot values in \nother instances referring the deleted instances will be set to null.",
                                                  "Delete Confirmation",
                                                  JOptionPane.YES_NO_OPTION);
        if (reply == JOptionPane.YES_OPTION) {
            XMLFileAdaptor fileAdaptor = PersistenceManager.getManager().getActiveFileAdaptor();
            
            // Create a _Deleted instance
            // These instances are used to keep track of
            // deletions.
            boolean isCreateDeleteInstanceOK = createDeleteInstance(fileAdaptor, 
                                                                    list,
                                                                    parentFrame);
            
            // Do the actual deletions
            if (isCreateDeleteInstanceOK) {
                for (GKInstance instance : list) {
                    fileAdaptor.deleteInstance(instance);
                }
            }
        }
    }
    
    public void delete(GKInstance instance,
                       JFrame parentFrame) {
        List<GKInstance> list = new ArrayList<GKInstance>(1);
        list.add(instance);
        delete(list, parentFrame);
    }
    
    /**
     * Given a list of instances that should be deleted, create a new
     * instance of class _Deleted and add it to the fileAdaptor.
     * This is used to track deletions.
     * 
     * N.B. if the user deletes a "_Deleted" instance, no new "_Deleted"
     * instance will be created.  Also, if the instance being deleted
     * was created locally (i.e. is not in the database), then no
     * "_Deleted" instance will be created.
     * 
     * Returns true if everything was OK, false if an error was
     * detected or if the user cancels.
     * 
     * @param dbAdaptor
     * @param instance
     */
    private boolean createDeleteInstance(XMLFileAdaptor fileAdaptor,
                                         List instances,
                                         JFrame parentFrame) {
        // Create an instance of class "_Deleted".  It's done
        // in this rather complicated way, rather than using
        // fileAdaptor.createNewInstance, because this latter
        // method inserts two copies of the instance into
        // fileAdaptor's cache.
        SchemaClass schemaClass;
        try {
            schemaClass = fileAdaptor.getSchema().getClassByName("_Deleted");
        } catch (Exception e1) {
            System.err.println("IdentifierDatabase.createDeleteInstance: problem getting schema class");
            e1.printStackTrace();
            return false;
        }
        Long dbID = fileAdaptor.getNextLocalID();
        GKInstance deleted = new GKInstance(schemaClass, dbID, fileAdaptor);

        // Add the DB_IDs of all instances to be deleted
        boolean foundValidInstance = false;
        String displayName = "Deletion of instances: ";
        for (Iterator it = instances.iterator(); it.hasNext();) {
            GKInstance instance = (GKInstance)it.next();
            
            if (instance.getSchemClass().getName().equals("_Deleted"))
                continue;
            if (instance.getDBID().intValue()<0)
                continue;
            
            if (foundValidInstance)
                displayName += ", ";
            
            foundValidInstance = true;
        
            Long deletedInstanceDB_ID = instance.getDBID();
            displayName += deletedInstanceDB_ID.toString();
            
            try {
                deleted.addAttributeValue("deletedInstanceDB_ID", new Integer(deletedInstanceDB_ID.intValue()));
            } catch (InvalidAttributeException e1) {
                System.err.println("CuratorActionCollection.createDeleteInstance: deletedInstanceDB_ID is a wrong name!");
                e1.printStackTrace();
            } catch (InvalidAttributeValueException e1) {
                System.err.println("CuratorActionCollection.createDeleteInstance: possibly a problem with deletedInstanceDB_ID=" + deletedInstanceDB_ID);
                e1.printStackTrace();
            }
        }
        
        if (foundValidInstance) {
            try {
                deleted.setAttributeValue("_displayName", displayName);
            } catch (InvalidAttributeException e) {
                System.err.println("CuratorActionCollection.createDeleteInstance: _displayName is a wrong name!");
                e.printStackTrace();
            } catch (InvalidAttributeValueException e) {
                System.err.println("CuratorActionCollection.createDeleteInstance: possibly a problem with displayName=" + displayName);
                e.printStackTrace();
            }

            // Let the curator add comment, etc.
            DeletedInstanceDialog deletedInstanceDialog = new DeletedInstanceDialog(parentFrame, deleted);
            deletedInstanceDialog.setModal(true);
            deletedInstanceDialog.setSize(400, 600);
            deletedInstanceDialog.setLocationRelativeTo(parentFrame);
            deletedInstanceDialog.setVisible(true);
            
            if (deletedInstanceDialog.isCancelClicked())
                return false;
            
            if (deletedInstanceDialog.isOKClicked())
                fileAdaptor.addNewInstance(deleted);
        }
        
        return true;
    }
}
